@@ -1,11 +1,29 @@
+"""CLI argument definitions for datamodel-codegen.
+
+Defines the ArgumentParser and all command-line options organized into groups:
+base options, typing customization, field customization, model customization,
+template customization, OpenAPI-specific options, and general options.
+"""
+
 from __future__ import annotations
 
+import json
 import locale
-from argparse import ArgumentParser, FileType, HelpFormatter, Namespace
+from argparse import ArgumentParser, ArgumentTypeError, BooleanOptionalAction, HelpFormatter, Namespace
 from operator import attrgetter
-from typing import TYPE_CHECKING
+from pathlib import Path
+from typing import TYPE_CHECKING, cast
 
-from datamodel_code_generator import DataModelType, InputFileType, OpenAPIScope
+from datamodel_code_generator import (
+    DEFAULT_SHARED_MODULE_NAME,
+    AllExportsCollisionStrategy,
+    AllExportsScope,
+    DataclassArguments,
+    DataModelType,
+    InputFileType,
+    OpenAPIScope,
+    ReuseScope,
+)
 from datamodel_code_generator.format import DatetimeClassType, Formatter, PythonVersion
 from datamodel_code_generator.model.pydantic_v2 import UnionMode
 from datamodel_code_generator.parser import LiteralType
@@ -20,15 +38,42 @@ DEFAULT_ENCODING = locale.getpreferredencoding()
 namespace = Namespace(no_color=False)
 
 
+def _dataclass_arguments(value: str) -> DataclassArguments:
+    """Parse JSON string and validate it as DataclassArguments."""
+    try:
+        result = json.loads(value)
+    except json.JSONDecodeError as e:
+        msg = f"Invalid JSON: {e}"
+        raise ArgumentTypeError(msg) from e
+    if not isinstance(result, dict):
+        msg = f"Expected a JSON dictionary, got {type(result).__name__}"
+        raise ArgumentTypeError(msg)
+    valid_keys = set(DataclassArguments.__annotations__.keys())
+    invalid_keys = set(result.keys()) - valid_keys
+    if invalid_keys:
+        msg = f"Invalid keys: {invalid_keys}. Valid keys are: {valid_keys}"
+        raise ArgumentTypeError(msg)
+    for key, val in result.items():
+        if not isinstance(val, bool):
+            msg = f"Expected bool for '{key}', got {type(val).__name__}"
+            raise ArgumentTypeError(msg)
+    return cast("DataclassArguments", result)
+
+
 class SortingHelpFormatter(HelpFormatter):
+    """Help formatter that sorts arguments and adds color to section headers."""
+
     def _bold_cyan(self, text: str) -> str:  # noqa: PLR6301
+        """Wrap text in ANSI bold cyan escape codes."""
         return f"\x1b[36;1m{text}\x1b[0m"
 
     def add_arguments(self, actions: Iterable[Action]) -> None:
+        """Add arguments sorted by option strings."""
         actions = sorted(actions, key=attrgetter("option_strings"))
         super().add_arguments(actions)
 
     def start_section(self, heading: str | None) -> None:
+        """Start a section with optional colored heading."""
         return super().start_section(heading if namespace.no_color or not heading else self._bold_cyan(heading))
 
 
@@ -118,6 +163,12 @@ model_options.add_argument(
     default=None,
     help="Models generated with a root-type field will be merged into the models using that root-type model",
 )
+model_options.add_argument(
+    "--skip-root-model",
+    action="store_true",
+    default=None,
+    help="Skip generating the model for the root schema element",
+)
 model_options.add_argument(
     "--disable-appending-item-suffix",
     help="Disable appending `Item` suffix to model name in an array",
@@ -166,12 +217,35 @@ model_options.add_argument(
     action="store_true",
     default=None,
 )
+model_options.add_argument(
+    "--dataclass-arguments",
+    type=_dataclass_arguments,
+    default=None,
+    help=(
+        "Custom dataclass arguments as a JSON dictionary, "
+        'e.g. \'{"frozen": true, "kw_only": true}\'. '
+        "Overrides --frozen-dataclasses and similar flags."
+    ),
+)
 model_options.add_argument(
     "--reuse-model",
     help="Reuse models on the field when a module has the model with the same content",
     action="store_true",
     default=None,
 )
+model_options.add_argument(
+    "--reuse-scope",
+    help="Scope for model reuse deduplication: module (per-file, default) or tree (cross-file with shared module). "
+    "Only effective when --reuse-model is set.",
+    choices=[s.value for s in ReuseScope],
+    default=None,
+)
+model_options.add_argument(
+    "--shared-module-name",
+    help=f'Name of the shared module for --reuse-scope=tree (default: "{DEFAULT_SHARED_MODULE_NAME}"). '
+    f'Use this option if your schema has a file named "{DEFAULT_SHARED_MODULE_NAME}".',
+    default=None,
+)
 model_options.add_argument(
     "--target-python-version",
     help="target python version",
@@ -221,6 +295,23 @@ model_options.add_argument(
     action="store_true",
     default=None,
 )
+model_options.add_argument(
+    "--all-exports-scope",
+    help="Generate __all__ in __init__.py with re-exports. "
+    "'children': export from direct child modules only. "
+    "'recursive': export from all descendant modules.",
+    choices=[s.value for s in AllExportsScope],
+    default=None,
+)
+model_options.add_argument(
+    "--all-exports-collision-strategy",
+    help="Strategy for name collisions when using --all-exports-scope=recursive. "
+    "'error': raise an error (default). "
+    "'minimal-prefix': add module prefix only to colliding names. "
+    "'full-prefix': add full module path prefix to colliding names.",
+    choices=[s.value for s in AllExportsCollisionStrategy],
+    default=None,
+)
 
 # ======================================================================================
 # Typing options for generated models
@@ -289,10 +380,16 @@ typing_options.add_argument(
 )
 typing_options.add_argument(
     "--use-subclass-enum",
-    help="Define Enum class as subclass with field type when enum has type (int, float, bytes, str)",
+    help="Define generic Enum class as subclass with field type when enum has type (int, float, bytes, str)",
     action="store_true",
     default=None,
 )
+typing_options.add_argument(
+    "--use-specialized-enum",
+    help="Don't use specialized Enum class (StrEnum, IntEnum) even if the target Python version supports it",
+    action=BooleanOptionalAction,
+    default=None,
+)
 typing_options.add_argument(
     "--use-union-operator",
     help="Use | operator for Union type (PEP 604).",
@@ -305,6 +402,28 @@ typing_options.add_argument(
     action="store_true",
     default=None,
 )
+typing_options.add_argument(
+    "--use-type-alias",
+    help="Use TypeAlias instead of root models (experimental)",
+    action="store_true",
+    default=None,
+)
+typing_options.add_argument(
+    "--disable-future-imports",
+    help="Disable __future__ imports",
+    action="store_true",
+    default=None,
+)
+typing_options.add_argument(
+    "--type-mappings",
+    help="Override default type mappings. "
+    'Format: "type+format=target" (e.g., "string+binary=string" to map binary format to string type) '
+    'or "format=target" (e.g., "binary=string"). '
+    "Can be specified multiple times.",
+    nargs="+",
+    type=str,
+    default=None,
+)
 
 # ======================================================================================
 # Customization options for generated model fields
@@ -391,6 +510,18 @@ field_options.add_argument(
     action="store_true",
     default=None,
 )
+field_options.add_argument(
+    "--use-attribute-docstrings",
+    help="Set use_attribute_docstrings=True in Pydantic v2 ConfigDict",
+    action="store_true",
+    default=None,
+)
+field_options.add_argument(
+    "--use-inline-field-description",
+    help="Use schema description to populate field docstring as inline docstring",
+    action="store_true",
+    default=None,
+)
 field_options.add_argument(
     "--union-mode",
     help="Union mode for only pydantic v2 field",
@@ -410,8 +541,13 @@ field_options.add_argument(
 # ======================================================================================
 template_options.add_argument(
     "--aliases",
-    help="Alias mapping file",
-    type=FileType("rt"),
+    help="Alias mapping file (JSON) for renaming fields. "
+    "Supports hierarchical formats: "
+    "Flat: {'field': 'alias'} applies to all occurrences. "
+    "Scoped: {'ClassName.field': 'alias'} applies to specific class. "
+    "Priority: scoped > flat. "
+    "Example: {'User.name': 'user_name', 'Address.name': 'addr_name', 'id': 'id_'}",
+    type=Path,
 )
 template_options.add_argument(
     "--custom-file-header",
@@ -437,8 +573,12 @@ template_options.add_argument(
 )
 template_options.add_argument(
     "--extra-template-data",
-    help="Extra template data",
-    type=FileType("rt"),
+    help="Extra template data for output models. Input is supposed to be a json/yaml file. "
+    "For OpenAPI and Jsonschema the keys are the spec path of the object, or the name of the object if you want to "
+    "apply the template data to multiple objects with the same name. "
+    "If you are using another input file type (e.g. GraphQL), the key is the name of the object. "
+    "The value is a dictionary of the template data to add.",
+    type=Path,
 )
 template_options.add_argument(
     "--use-double-quotes",
@@ -475,7 +615,7 @@ base_options.add_argument(
 template_options.add_argument(
     "--custom-formatters-kwargs",
     help="A file with kwargs for custom formatters.",
-    type=FileType("rt"),
+    type=Path,
 )
 
 # ======================================================================================
@@ -517,6 +657,14 @@ openapi_options.add_argument(
 # ======================================================================================
 # General options
 # ======================================================================================
+general_options.add_argument(
+    "--check",
+    action="store_true",
+    default=None,
+    help="Verify generated files are up-to-date without modifying them. "
+    "Exits with code 1 if differences found, 0 if up-to-date. "
+    "Useful for CI to ensure generated code is committed.",
+)
 general_options.add_argument(
     "--debug",
     help="show debug message (require \"debug\". `$ pip install 'datamodel-code-generator[debug]'`)",
@@ -542,6 +690,18 @@ general_options.add_argument(
     default=False,
     help="disable colorized output",
 )
+general_options.add_argument(
+    "--generate-pyproject-config",
+    action="store_true",
+    default=None,
+    help="Generate pyproject.toml configuration from the provided CLI arguments and exit",
+)
+general_options.add_argument(
+    "--generate-cli-command",
+    action="store_true",
+    default=None,
+    help="Generate CLI command from pyproject.toml configuration and exit",
+)
 general_options.add_argument(
     "--version",
     action="store_true",
