@@ -1,3 +1,8 @@
+"""Pydantic v1 type manager.
+
+Maps schema types to Pydantic v1 specific types (constr, conint, AnyUrl, etc.).
+"""
+
 from __future__ import annotations
 
 from decimal import Decimal
@@ -62,8 +67,8 @@ def type_map_factory(
     strict_types: Sequence[StrictTypes],
     pattern_key: str,
     use_pendulum: bool,  # noqa: FBT001
-    target_datetime_class: DatetimeClassType,  # noqa: ARG001
 ) -> dict[Types, DataType]:
+    """Create a mapping of schema types to Pydantic v1 data types."""
     data_type_int = data_type(type="int")
     data_type_float = data_type(type="float")
     data_type_str = data_type(type="str")
@@ -122,6 +127,7 @@ def type_map_factory(
 
 
 def strict_type_map_factory(data_type: type[DataType]) -> dict[StrictTypes, DataType]:
+    """Create a mapping of strict types to Pydantic v1 strict data types."""
     return {
         StrictTypes.int: data_type.from_import(IMPORT_STRICT_INT, strict=True),
         StrictTypes.float: data_type.from_import(IMPORT_STRICT_FLOAT, strict=True),
@@ -154,6 +160,8 @@ escape_characters = str.maketrans({
 
 
 class DataTypeManager(_DataTypeManager):
+    """Manage data type mappings for Pydantic v1 models."""
+
     PATTERN_KEY: ClassVar[str] = "regex"
 
     def __init__(  # noqa: PLR0913, PLR0917
@@ -168,6 +176,7 @@ class DataTypeManager(_DataTypeManager):
         target_datetime_class: DatetimeClassType | None = None,
         treat_dot_as_module: bool = False,  # noqa: FBT001, FBT002
     ) -> None:
+        """Initialize the DataTypeManager with Pydantic v1 type mappings."""
         super().__init__(
             python_version,
             use_standard_collections,
@@ -208,17 +217,18 @@ class DataTypeManager(_DataTypeManager):
         data_type: type[DataType],
         strict_types: Sequence[StrictTypes],
         pattern_key: str,
-        target_datetime_class: DatetimeClassType,  # noqa: ARG002
+        target_datetime_class: DatetimeClassType | None,  # noqa: ARG002
     ) -> dict[Types, DataType]:
+        """Create type mapping with Pydantic v1 specific types."""
         return type_map_factory(
             data_type,
             strict_types,
             pattern_key,
             self.use_pendulum,
-            self.target_datetime_class,
         )
 
     def transform_kwargs(self, kwargs: dict[str, Any], filter_: set[str]) -> dict[str, str]:
+        """Transform schema kwargs to Pydantic v1 field kwargs."""
         return {self.kwargs_schema_to_model.get(k, k): v for (k, v) in kwargs.items() if v is not None and k in filter_}
 
     def get_data_int_type(  # noqa: PLR0911
@@ -226,6 +236,7 @@ class DataTypeManager(_DataTypeManager):
         types: Types,
         **kwargs: Any,
     ) -> DataType:
+        """Get int data type with constraints (conint, PositiveInt, etc.)."""
         data_type_kwargs: dict[str, Any] = self.transform_kwargs(kwargs, number_kwargs)
         strict = StrictTypes.int in self.strict_types
         if data_type_kwargs:
@@ -251,6 +262,7 @@ class DataTypeManager(_DataTypeManager):
         types: Types,
         **kwargs: Any,
     ) -> DataType:
+        """Get float data type with constraints (confloat, PositiveFloat, etc.)."""
         data_type_kwargs = self.transform_kwargs(kwargs, number_kwargs)
         strict = StrictTypes.float in self.strict_types
         if data_type_kwargs:
@@ -272,6 +284,7 @@ class DataTypeManager(_DataTypeManager):
         return self.type_map[types]
 
     def get_data_decimal_type(self, types: Types, **kwargs: Any) -> DataType:
+        """Get decimal data type with constraints (condecimal)."""
         data_type_kwargs = self.transform_kwargs(kwargs, number_kwargs)
         if data_type_kwargs:
             return self.data_type.from_import(
@@ -281,6 +294,7 @@ class DataTypeManager(_DataTypeManager):
         return self.type_map[types]
 
     def get_data_str_type(self, types: Types, **kwargs: Any) -> DataType:
+        """Get string data type with constraints (constr)."""
         data_type_kwargs: dict[str, Any] = self.transform_kwargs(kwargs, string_kwargs)
         strict = StrictTypes.str in self.strict_types
         if data_type_kwargs:
@@ -296,6 +310,7 @@ class DataTypeManager(_DataTypeManager):
         return self.type_map[types]
 
     def get_data_bytes_type(self, types: Types, **kwargs: Any) -> DataType:
+        """Get bytes data type with constraints (conbytes)."""
         data_type_kwargs: dict[str, Any] = self.transform_kwargs(kwargs, bytes_kwargs)
         strict = StrictTypes.bytes in self.strict_types
         if data_type_kwargs and not strict:
@@ -311,6 +326,7 @@ class DataTypeManager(_DataTypeManager):
         types: Types,
         **kwargs: Any,
     ) -> DataType:
+        """Get data type with appropriate constraints for the given type."""
         if types == Types.string:
             return self.get_data_str_type(types, **kwargs)
         if types in {Types.int32, Types.int64, Types.integer}:
