@@ -1,3 +1,9 @@
+"""OpenAPI and Swagger specification parser.
+
+Extends JsonSchemaParser to handle OpenAPI 2.0 (Swagger), 3.0, and 3.1
+specifications, including paths, operations, parameters, and request/response bodies.
+"""
+
 from __future__ import annotations
 
 import re
@@ -10,25 +16,28 @@ from warnings import warn
 from pydantic import Field
 
 from datamodel_code_generator import (
+    DEFAULT_SHARED_MODULE_NAME,
+    DataclassArguments,
     Error,
     LiteralType,
     OpenAPIScope,
     PythonVersion,
     PythonVersionMin,
-    load_yaml,
+    ReuseScope,
+    YamlValue,
+    load_yaml_dict,
     snooper_to_methods,
 )
 from datamodel_code_generator.format import DEFAULT_FORMATTERS, DatetimeClassType, Formatter
 from datamodel_code_generator.model import DataModel, DataModelFieldBase
 from datamodel_code_generator.model import pydantic as pydantic_model
-from datamodel_code_generator.parser import DefaultPutDict  # noqa: TC001 # needed for type check
 from datamodel_code_generator.parser.base import get_special_path
 from datamodel_code_generator.parser.jsonschema import (
     JsonSchemaObject,
     JsonSchemaParser,
     get_model_by_path,
 )
-from datamodel_code_generator.reference import snake_to_upper_camel
+from datamodel_code_generator.reference import FieldNameResolver, snake_to_upper_camel
 from datamodel_code_generator.types import (
     DataType,
     DataTypeManager,
@@ -42,6 +51,8 @@ if TYPE_CHECKING:
     from pathlib import Path
     from urllib.parse import ParseResult
 
+    from datamodel_code_generator.parser import DefaultPutDict
+
 
 RE_APPLICATION_JSON_PATTERN: Pattern[str] = re.compile(r"^application/.*json$")
 
@@ -58,6 +69,8 @@ OPERATION_NAMES: list[str] = [
 
 
 class ParameterLocation(Enum):
+    """Represent OpenAPI parameter locations."""
+
     query = "query"
     header = "header"
     path = "path"
@@ -68,57 +81,73 @@ BaseModelT = TypeVar("BaseModelT", bound=BaseModel)
 
 
 class ReferenceObject(BaseModel):
+    """Represent an OpenAPI reference object ($ref)."""
+
     ref: str = Field(..., alias="$ref")
 
 
 class ExampleObject(BaseModel):
+    """Represent an OpenAPI example object."""
+
     summary: Optional[str] = None  # noqa: UP045
     description: Optional[str] = None  # noqa: UP045
-    value: Any = None
+    value: YamlValue = None
     externalValue: Optional[str] = None  # noqa: N815, UP045
 
 
 class MediaObject(BaseModel):
+    """Represent an OpenAPI media type object."""
+
     schema_: Optional[Union[ReferenceObject, JsonSchemaObject]] = Field(None, alias="schema")  # noqa: UP007, UP045
-    example: Any = None
+    example: YamlValue = None
     examples: Optional[Union[str, ReferenceObject, ExampleObject]] = None  # noqa: UP007, UP045
 
 
 class ParameterObject(BaseModel):
+    """Represent an OpenAPI parameter object."""
+
     name: Optional[str] = None  # noqa: UP045
     in_: Optional[ParameterLocation] = Field(None, alias="in")  # noqa: UP045
     description: Optional[str] = None  # noqa: UP045
     required: bool = False
     deprecated: bool = False
     schema_: Optional[JsonSchemaObject] = Field(None, alias="schema")  # noqa: UP045
-    example: Any = None
+    example: YamlValue = None
     examples: Optional[Union[str, ReferenceObject, ExampleObject]] = None  # noqa: UP007, UP045
     content: dict[str, MediaObject] = {}  # noqa: RUF012
 
 
 class HeaderObject(BaseModel):
+    """Represent an OpenAPI header object."""
+
     description: Optional[str] = None  # noqa: UP045
     required: bool = False
     deprecated: bool = False
     schema_: Optional[JsonSchemaObject] = Field(None, alias="schema")  # noqa: UP045
-    example: Any = None
+    example: YamlValue = None
     examples: Optional[Union[str, ReferenceObject, ExampleObject]] = None  # noqa: UP007, UP045
     content: dict[str, MediaObject] = {}  # noqa: RUF012
 
 
 class RequestBodyObject(BaseModel):
+    """Represent an OpenAPI request body object."""
+
     description: Optional[str] = None  # noqa: UP045
     content: dict[str, MediaObject] = {}  # noqa: RUF012
     required: bool = False
 
 
 class ResponseObject(BaseModel):
+    """Represent an OpenAPI response object."""
+
     description: Optional[str] = None  # noqa: UP045
     headers: dict[str, ParameterObject] = {}  # noqa: RUF012
     content: dict[Union[str, int], MediaObject] = {}  # noqa: RUF012, UP007
 
 
 class Operation(BaseModel):
+    """Represent an OpenAPI operation object."""
+
     tags: list[str] = []  # noqa: RUF012
     summary: Optional[str] = None  # noqa: UP045
     description: Optional[str] = None  # noqa: UP045
@@ -130,6 +159,8 @@ class Operation(BaseModel):
 
 
 class ComponentsObject(BaseModel):
+    """Represent an OpenAPI components object."""
+
     schemas: dict[str, Union[ReferenceObject, JsonSchemaObject]] = {}  # noqa: RUF012, UP007
     responses: dict[str, Union[ReferenceObject, ResponseObject]] = {}  # noqa: RUF012, UP007
     examples: dict[str, Union[ReferenceObject, ExampleObject]] = {}  # noqa: RUF012, UP007
@@ -139,6 +170,8 @@ class ComponentsObject(BaseModel):
 
 @snooper_to_methods()
 class OpenAPIParser(JsonSchemaParser):
+    """Parser for OpenAPI 2.0/3.0/3.1 and Swagger specifications."""
+
     SCHEMA_PATHS: ClassVar[list[str]] = ["#/components/schemas"]
 
     def __init__(  # noqa: PLR0913
@@ -170,13 +203,18 @@ class OpenAPIParser(JsonSchemaParser):
         base_path: Path | None = None,
         use_schema_description: bool = False,
         use_field_description: bool = False,
+        use_attribute_docstrings: bool = False,
+        use_inline_field_description: bool = False,
         use_default_kwarg: bool = False,
         reuse_model: bool = False,
+        reuse_scope: ReuseScope | None = None,
+        shared_module_name: str = DEFAULT_SHARED_MODULE_NAME,
         encoding: str = "utf-8",
         enum_field_as_literal: LiteralType | None = None,
         use_one_literal_as_default: bool = False,
         set_default_enum_member: bool = False,
         use_subclass_enum: bool = False,
+        use_specialized_enum: bool = True,
         strict_nullable: bool = False,
         use_generic_container_types: bool = False,
         enable_faux_immutability: bool = False,
@@ -203,6 +241,8 @@ class OpenAPIParser(JsonSchemaParser):
         use_union_operator: bool = False,
         allow_responses_without_content: bool = False,
         collapse_root_models: bool = False,
+        skip_root_model: bool = False,
+        use_type_alias: bool = False,
         special_field_name_prefix: str | None = None,
         remove_special_field_name_prefix: bool = False,
         capitalise_enum_members: bool = False,
@@ -215,13 +255,17 @@ class OpenAPIParser(JsonSchemaParser):
         treat_dot_as_module: bool = False,
         use_exact_imports: bool = False,
         default_field_extras: dict[str, Any] | None = None,
-        target_datetime_class: DatetimeClassType = DatetimeClassType.Datetime,
+        target_datetime_class: DatetimeClassType | None = None,
         keyword_only: bool = False,
         frozen_dataclasses: bool = False,
         no_alias: bool = False,
         formatters: list[Formatter] = DEFAULT_FORMATTERS,
         parent_scoped_naming: bool = False,
+        dataclass_arguments: DataclassArguments | None = None,
+        type_mappings: list[str] | None = None,
     ) -> None:
+        """Initialize the OpenAPI parser with extensive configuration options."""
+        target_datetime_class = target_datetime_class or DatetimeClassType.Awaredatetime
         super().__init__(
             source=source,
             data_model_type=data_model_type,
@@ -249,13 +293,18 @@ class OpenAPIParser(JsonSchemaParser):
             base_path=base_path,
             use_schema_description=use_schema_description,
             use_field_description=use_field_description,
+            use_attribute_docstrings=use_attribute_docstrings,
+            use_inline_field_description=use_inline_field_description,
             use_default_kwarg=use_default_kwarg,
             reuse_model=reuse_model,
+            reuse_scope=reuse_scope,
+            shared_module_name=shared_module_name,
             encoding=encoding,
             enum_field_as_literal=enum_field_as_literal,
             use_one_literal_as_default=use_one_literal_as_default,
             set_default_enum_member=set_default_enum_member,
             use_subclass_enum=use_subclass_enum,
+            use_specialized_enum=use_specialized_enum,
             strict_nullable=strict_nullable,
             use_generic_container_types=use_generic_container_types,
             enable_faux_immutability=enable_faux_immutability,
@@ -280,6 +329,8 @@ class OpenAPIParser(JsonSchemaParser):
             use_union_operator=use_union_operator,
             allow_responses_without_content=allow_responses_without_content,
             collapse_root_models=collapse_root_models,
+            skip_root_model=skip_root_model,
+            use_type_alias=use_type_alias,
             special_field_name_prefix=special_field_name_prefix,
             remove_special_field_name_prefix=remove_special_field_name_prefix,
             capitalise_enum_members=capitalise_enum_members,
@@ -298,16 +349,22 @@ class OpenAPIParser(JsonSchemaParser):
             no_alias=no_alias,
             formatters=formatters,
             parent_scoped_naming=parent_scoped_naming,
+            dataclass_arguments=dataclass_arguments,
+            type_mappings=type_mappings,
         )
         self.open_api_scopes: list[OpenAPIScope] = openapi_scopes or [OpenAPIScope.Schemas]
         self.include_path_parameters: bool = include_path_parameters
+        self._discriminator_schemas: dict[str, dict[str, Any]] = {}
+        self._discriminator_subtypes: dict[str, list[str]] = defaultdict(list)
 
     def get_ref_model(self, ref: str) -> dict[str, Any]:
+        """Resolve a reference to its model definition."""
         ref_file, ref_path = self.model_resolver.resolve_ref(ref).split("#", 1)
         ref_body = self._get_ref_body(ref_file) if ref_file else self.raw_obj
         return get_model_by_path(ref_body, ref_path.split("/")[1:])
 
     def get_data_type(self, obj: JsonSchemaObject) -> DataType:
+        """Get data type from JSON schema object, handling OpenAPI nullable semantics."""
         # OpenAPI 3.0 doesn't allow `null` in the `type` field and list of types
         # https://swagger.io/docs/specification/data-models/data-types/#null
         # OpenAPI 3.1 does allow `null` in the `type` field and is equivalent to
@@ -317,7 +374,52 @@ class OpenAPIParser(JsonSchemaParser):
 
         return super().get_data_type(obj)
 
+    def _get_discriminator_union_type(self, ref: str) -> DataType | None:
+        """Create a union type for discriminator subtypes if available."""
+        subtypes = self._discriminator_subtypes.get(ref, [])
+        if not subtypes:
+            return None
+        refs = map(self.model_resolver.add_ref, subtypes)
+        return self.data_type(data_types=[self.data_type(reference=r) for r in refs])
+
+    def get_ref_data_type(self, ref: str) -> DataType:
+        """Get data type for a reference, handling discriminator polymorphism."""
+        if ref in self._discriminator_schemas and (union_type := self._get_discriminator_union_type(ref)):
+            return union_type
+        return super().get_ref_data_type(ref)
+
+    def parse_object_fields(
+        self,
+        obj: JsonSchemaObject,
+        path: list[str],
+        module_name: Optional[str] = None,  # noqa: UP045
+        class_name: Optional[str] = None,  # noqa: UP045
+    ) -> list[DataModelFieldBase]:
+        """Parse object fields, adding discriminator info for allOf polymorphism."""
+        fields = super().parse_object_fields(obj, path, module_name, class_name=class_name)
+        properties = obj.properties or {}
+
+        result_fields: list[DataModelFieldBase] = []
+        for field_obj in fields:
+            field = properties.get(field_obj.original_name)
+
+            if (
+                isinstance(field, JsonSchemaObject)
+                and field.ref
+                and (discriminator := self._discriminator_schemas.get(field.ref))
+            ):
+                new_field_type = self._get_discriminator_union_type(field.ref) or field_obj.data_type
+                field_obj = self.data_model_field_type(**{  # noqa: PLW2901
+                    **field_obj.__dict__,
+                    "data_type": new_field_type,
+                    "extras": {**field_obj.extras, "discriminator": discriminator},
+                })
+            result_fields.append(field_obj)
+
+        return result_fields
+
     def resolve_object(self, obj: ReferenceObject | BaseModelT, object_type: type[BaseModelT]) -> BaseModelT:
+        """Resolve a reference object to its actual type or return the object as-is."""
         if isinstance(obj, ReferenceObject):
             ref_obj = self.get_ref_model(obj.ref)
             return object_type.parse_obj(ref_obj)
@@ -329,6 +431,7 @@ class OpenAPIParser(JsonSchemaParser):
         obj: JsonSchemaObject,
         path: list[str],
     ) -> DataType:
+        """Parse a JSON schema object into a data type."""
         if obj.is_array:
             data_type = self.parse_array(name, obj, [*path, name])
         elif obj.allOf:  # pragma: no cover
@@ -354,6 +457,7 @@ class OpenAPIParser(JsonSchemaParser):
         request_body: RequestBodyObject,
         path: list[str],
     ) -> dict[str, DataType]:
+        """Parse request body content into data types by media type."""
         data_types: dict[str, DataType] = {}
         for (
             media_type,
@@ -362,6 +466,7 @@ class OpenAPIParser(JsonSchemaParser):
             if isinstance(media_obj.schema_, JsonSchemaObject):
                 data_types[media_type] = self.parse_schema(name, media_obj.schema_, [*path, media_type])
             elif media_obj.schema_ is not None:
+                self.resolve_ref(media_obj.schema_.ref)
                 data_types[media_type] = self.get_ref_data_type(media_obj.schema_.ref)
         return data_types
 
@@ -371,6 +476,7 @@ class OpenAPIParser(JsonSchemaParser):
         responses: dict[str | int, ReferenceObject | ResponseObject],
         path: list[str],
     ) -> dict[str | int, dict[str, DataType]]:
+        """Parse response objects into data types by status code and content type."""
         data_types: defaultdict[str | int, dict[str, DataType]] = defaultdict(dict)
         for status_code, detail in responses.items():
             if isinstance(detail, ReferenceObject):
@@ -395,6 +501,7 @@ class OpenAPIParser(JsonSchemaParser):
                         [*path, str(status_code), content_type],  # pyright: ignore[reportArgumentType]
                     )
                 else:
+                    self.resolve_ref(object_schema.ref)
                     data_types[status_code][content_type] = self.get_ref_data_type(  # pyright: ignore[reportArgumentType]
                         object_schema.ref
                     )
@@ -408,11 +515,15 @@ class OpenAPIParser(JsonSchemaParser):
         tags: list[str],
         path: list[str],  # noqa: ARG003
     ) -> list[str]:
+        """Parse operation tags."""
         return tags
 
+    _field_name_resolver: FieldNameResolver = FieldNameResolver()
+
     @classmethod
     def _get_model_name(cls, path_name: str, method: str, suffix: str) -> str:
-        camel_path_name = snake_to_upper_camel(path_name.replace("/", "_"))
+        normalized = cls._field_name_resolver.get_valid_name(path_name, ignore_snake_case_field=True)
+        camel_path_name = snake_to_upper_camel(normalized)
         return f"{camel_path_name}{method.capitalize()}{suffix}"
 
     def parse_all_parameters(
@@ -421,6 +532,7 @@ class OpenAPIParser(JsonSchemaParser):
         parameters: list[ReferenceObject | ParameterObject],
         path: list[str],
     ) -> DataType | None:
+        """Parse all operation parameters into a data model."""
         fields: list[DataModelFieldBase] = []
         exclude_field_names: set[str] = set()
         reference = self.model_resolver.add(path, name, class_name=True, unique=True)
@@ -439,7 +551,9 @@ class OpenAPIParser(JsonSchemaParser):
                 raise Exception(msg)  # noqa: TRY002
 
             field_name, alias = self.model_resolver.get_valid_field_name_and_alias(
-                field_name=parameter_name, excludes=exclude_field_names
+                field_name=parameter_name,
+                excludes=exclude_field_names,
+                class_name=name,
             )
             if parameter.schema_:
                 fields.append(
@@ -495,6 +609,7 @@ class OpenAPIParser(JsonSchemaParser):
                         extras=self.get_field_extras(object_schema) if object_schema else {},
                         use_annotated=self.use_annotated,
                         use_field_description=self.use_field_description,
+                        use_inline_field_description=self.use_inline_field_description,
                         use_default_kwarg=self.use_default_kwarg,
                         original_name=parameter_name,
                         has_default=object_schema.has_default if object_schema else False,
@@ -513,6 +628,7 @@ class OpenAPIParser(JsonSchemaParser):
                     custom_template_dir=self.custom_template_dir,
                     keyword_only=self.keyword_only,
                     treat_dot_as_module=self.treat_dot_as_module,
+                    dataclass_arguments=self.dataclass_arguments,
                 )
             )
             return self.data_type(reference=reference)
@@ -524,6 +640,7 @@ class OpenAPIParser(JsonSchemaParser):
         raw_operation: dict[str, Any],
         path: list[str],
     ) -> None:
+        """Parse an OpenAPI operation including parameters, request body, and responses."""
         operation = Operation.parse_obj(raw_operation)
         path_name, method = path[-2:]
         if self.use_operation_id_as_name:
@@ -565,7 +682,8 @@ class OpenAPIParser(JsonSchemaParser):
                 path=[*path, "tags"],
             )
 
-    def parse_raw(self) -> None:  # noqa: PLR0912
+    def parse_raw(self) -> None:  # noqa: PLR0912, PLR0915
+        """Parse OpenAPI specification including schemas, paths, and operations."""
         for source, path_parts in self._get_context_source_path_parts():  # noqa: PLR1702
             if self.validation:
                 warn(
@@ -591,22 +709,20 @@ class OpenAPIParser(JsonSchemaParser):
                         stacklevel=2,
                     )
 
-            specification: dict[str, Any] = load_yaml(source.text)
+            specification: dict[str, Any] = load_yaml_dict(source.text)
             self.raw_obj = specification
-            schemas: dict[Any, Any] = specification.get("components", {}).get("schemas", {})
+            self._collect_discriminator_schemas()
+            schemas: dict[str, Any] = specification.get("components", {}).get("schemas", {})
             security: list[dict[str, list[str]]] | None = specification.get("security")
             if OpenAPIScope.Schemas in self.open_api_scopes:
-                for (
-                    obj_name,
-                    raw_obj,
-                ) in schemas.items():
+                for obj_name, raw_obj in schemas.items():
                     self.parse_raw_obj(
                         obj_name,
                         raw_obj,
                         [*path_parts, "#/components", "schemas", obj_name],
                     )
             if OpenAPIScope.Paths in self.open_api_scopes:
-                paths: dict[str, dict[str, Any]] = specification.get("paths", {})
+                paths: dict[str, Any] = specification.get("paths", {})
                 parameters: list[dict[str, Any]] = [
                     self._get_ref_body(p["$ref"]) if "$ref" in p else p
                     for p in paths.get("parameters", [])
@@ -639,4 +755,58 @@ class OpenAPIParser(JsonSchemaParser):
                             [*path, operation_name],
                         )
 
+            if OpenAPIScope.Webhooks in self.open_api_scopes:
+                webhooks: dict[str, dict[str, Any]] = specification.get("webhooks", {})
+                webhooks_path = [*path_parts, "#/webhooks"]
+                for webhook_name, methods_ in webhooks.items():
+                    methods = self.get_ref_model(methods_["$ref"]) if "$ref" in methods_ else methods_
+                    webhook_parameters: list[dict[str, Any]] = []
+                    if "parameters" in methods:
+                        webhook_parameters = [
+                            self._get_ref_body(p["$ref"]) if "$ref" in p else p
+                            for p in methods["parameters"]
+                            if isinstance(p, dict)
+                        ]
+                    relative_webhook_name = webhook_name.removeprefix("/")
+                    if relative_webhook_name:
+                        path = [*webhooks_path, relative_webhook_name]
+                    else:  # pragma: no cover
+                        path = get_special_path("root", webhooks_path)
+                    for operation_name, raw_operation in methods.items():
+                        if operation_name not in OPERATION_NAMES:
+                            continue
+                        if webhook_parameters:
+                            if "parameters" in raw_operation:
+                                raw_operation["parameters"].extend(webhook_parameters)
+                            else:
+                                raw_operation["parameters"] = webhook_parameters.copy()
+                        if security is not None and "security" not in raw_operation:
+                            raw_operation["security"] = security
+                        self.parse_operation(
+                            raw_operation,
+                            [*path, operation_name],
+                        )
+
         self._resolve_unparsed_json_pointer()
+
+    def _collect_discriminator_schemas(self) -> None:
+        """Collect schemas with discriminators but no oneOf/anyOf, and find their subtypes."""
+        schemas: dict[str, Any] = self.raw_obj.get("components", {}).get("schemas", {})
+
+        for schema_name, schema in schemas.items():
+            discriminator = schema.get("discriminator")
+            if not discriminator:
+                continue
+
+            if schema.get("oneOf") or schema.get("anyOf"):
+                continue
+
+            ref = f"#/components/schemas/{schema_name}"
+            self._discriminator_schemas[ref] = discriminator
+
+        for schema_name, schema in schemas.items():
+            for all_of_item in schema.get("allOf", []):
+                ref_in_allof = all_of_item.get("$ref")
+                if ref_in_allof and ref_in_allof in self._discriminator_schemas:
+                    subtype_ref = f"#/components/schemas/{schema_name}"
+                    self._discriminator_subtypes[ref_in_allof].append(subtype_ref)
