@@ -1,3 +1,5 @@
+"""Tests for OpenAPI/Swagger schema parser."""
+
 from __future__ import annotations
 
 import os
@@ -15,7 +17,14 @@ from datamodel_code_generator.model import DataModelFieldBase
 from datamodel_code_generator.model.pydantic import DataModelField
 from datamodel_code_generator.parser.base import dump_templates
 from datamodel_code_generator.parser.jsonschema import JsonSchemaObject
-from datamodel_code_generator.parser.openapi import OpenAPIParser
+from datamodel_code_generator.parser.openapi import (
+    MediaObject,
+    OpenAPIParser,
+    ParameterObject,
+    RequestBodyObject,
+    ResponseObject,
+)
+from tests.conftest import assert_output, assert_parser_modules, assert_parser_results
 
 DATA_PATH: Path = Path(__file__).parents[1] / "data" / "openapi"
 
@@ -29,6 +38,7 @@ def get_expected_file(
     base_class: str | None = None,
     prefix: str | None = None,
 ) -> Path:
+    """Get expected output file path for test."""
     params: list[str] = []
     if with_import:
         params.append("with_import")
@@ -121,11 +131,12 @@ class Pets(BaseModel):
         (
             {"properties": {"kind": {"type": "array", "items": []}}},
             """class Pets(BaseModel):
-    kind: Optional[List] = None""",
+    kind: Optional[List[Any]] = None""",
         ),
     ],
 )
 def test_parse_object(source_obj: dict[str, Any], generated_classes: str) -> None:
+    """Test parsing OpenAPI object schemas."""
     parser = OpenAPIParser("")
     parser.parse_object("Pets", JsonSchemaObject.parse_obj(source_obj), [])
     assert dump_templates(list(parser.results)) == generated_classes
@@ -164,11 +175,12 @@ class Pets(BaseModel):
                 "items": {},
             },
             """class Pets(BaseModel):
-    __root__: List""",
+    __root__: List[Any]""",
         ),
     ],
 )
 def test_parse_array(source_obj: dict[str, Any], generated_classes: str) -> None:
+    """Test parsing OpenAPI array schemas."""
     parser = OpenAPIParser("")
     parser.parse_array("Pets", JsonSchemaObject.parse_obj(source_obj), [])
     assert dump_templates(list(parser.results)) == generated_classes
@@ -196,16 +208,14 @@ def test_parse_array(source_obj: dict[str, Any], generated_classes: str) -> None
     ],
 )
 def test_openapi_parser_parse(with_import: bool, format_: bool, base_class: str | None) -> None:
+    """Test OpenAPI parser with various configurations."""
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
         source=Path(DATA_PATH / "api.yaml"),
         base_class=base_class,
     )
     expected_file = get_expected_file("openapi_parser_parse", with_import, format_, base_class)
-    assert (
-        parser.parse(with_import=with_import, format_=format_, settings_path=DATA_PATH.parent)
-        == expected_file.read_text()
-    )
+    assert_output(parser.parse(with_import=with_import, format_=format_, settings_path=DATA_PATH.parent), expected_file)
 
 
 @pytest.mark.parametrize(
@@ -224,39 +234,42 @@ def test_openapi_parser_parse(with_import: bool, format_: bool, base_class: str
     ],
 )
 def test_parse_root_type(source_obj: dict[str, Any], generated_classes: str) -> None:
+    """Test parsing OpenAPI root type schemas."""
     parser = OpenAPIParser("")
     parser.parse_root_type("Name", JsonSchemaObject.parse_obj(source_obj), [])
     assert dump_templates(list(parser.results)) == generated_classes
 
 
 def test_openapi_parser_parse_duplicate_models(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with duplicate model names."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "duplicate_models.yaml"),
     )
-    assert (
-        parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_duplicate_models" / "output.py").read_text()
-    )
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_duplicate_models" / "output.py")
 
 
 def test_openapi_parser_parse_duplicate_model_with_simplify(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with duplicate models and simplification."""
     monkeypatch.chdir(tmp_path)
     raw = Path(DATA_PATH / "duplicate_model_simplify.yaml")
     parser = OpenAPIParser(raw)
-    expected = (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_duplicate_models_simplify" / "output.py").read_text()
-    got = parser.parse()
-    assert got == expected
+    assert_output(
+        parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_duplicate_models_simplify" / "output.py"
+    )
 
 
 def test_openapi_parser_parse_resolved_models(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with resolved model references."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "resolved_models.yaml"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_resolved_models" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_resolved_models" / "output.py")
 
 
 def test_openapi_parser_parse_lazy_resolved_models(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with lazy resolved model references."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "lazy_resolved_models.yaml"),
@@ -302,6 +315,7 @@ class Results(BaseModel):
 
 
 def test_openapi_parser_parse_x_enum_varnames(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with x-enum-varnames extension."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "x_enum_varnames.yaml"),
@@ -356,87 +370,106 @@ class UnknownTypeNumber(Enum):
 
 @pytest.mark.skipif(pydantic.VERSION < "1.9.0", reason="Require Pydantic version 1.9.0 or later ")
 def test_openapi_parser_parse_enum_models() -> None:
+    """Test parsing OpenAPI enum models."""
     parser = OpenAPIParser(
         Path(DATA_PATH / "enum_models.yaml").read_text(encoding="utf-8"),
         target_python_version=PythonVersionMin,
     )
     expected_dir = EXPECTED_OPEN_API_PATH / "openapi_parser_parse_enum_models"
-    assert parser.parse() == (expected_dir / "output.py").read_text()
+    assert_output(parser.parse(), expected_dir / "output.py")
 
 
 def test_openapi_parser_parse_anyof(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with anyOf schemas."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "anyof.yaml"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_anyof" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_anyof" / "output.py")
+
+
+def test_openapi_parser_parse_anyof_required(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with anyOf and required fields."""
+    monkeypatch.chdir(tmp_path)
+    parser = OpenAPIParser(
+        Path(DATA_PATH / "anyof_required.yaml"),
+    )
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_anyof_required" / "output.py")
 
 
 def test_openapi_parser_parse_nested_anyof(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with nested anyOf schemas."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "nested_anyof.yaml").read_text(encoding="utf-8"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_nested_anyof" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_nested_anyof" / "output.py")
 
 
 def test_openapi_parser_parse_oneof(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with oneOf schemas."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "oneof.yaml"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_oneof" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_oneof" / "output.py")
 
 
 def test_openapi_parser_parse_nested_oneof(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with nested oneOf schemas."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "nested_oneof.yaml").read_text(encoding="utf-8"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_nested_oneof" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_nested_oneof" / "output.py")
 
 
 def test_openapi_parser_parse_allof_ref(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with allOf references."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "allof_same_prefix_with_ref.yaml"),
     )
-    assert (
-        parser.parse()
-        == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_allof_same_prefix_with_ref" / "output.py").read_text()
+    assert_output(
+        parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_allof_same_prefix_with_ref" / "output.py"
     )
 
 
 def test_openapi_parser_parse_allof(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with allOf schemas."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "allof.yaml"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_allof" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_allof" / "output.py")
+
+
+def test_openapi_parser_parse_allof_required_fields(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with allOf and required fields."""
+    monkeypatch.chdir(tmp_path)
+    parser = OpenAPIParser(
+        Path(DATA_PATH / "allof_required_fields.yaml"),
+    )
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_allof_required_fields" / "output.py")
 
 
 def test_openapi_parser_parse_alias(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with field aliases."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         Path(DATA_PATH / "alias.yaml"),
     )
     delimiter = "\\" if platform.system() == "Windows" else "/"
     results = {delimiter.join(p): r for p, r in parser.parse().items()}
-    openapi_parser_parse_alias_dir = EXPECTED_OPEN_API_PATH / "openapi_parser_parse_alias"
-    for path in openapi_parser_parse_alias_dir.rglob("*.py"):
-        key = str(path.relative_to(openapi_parser_parse_alias_dir))
-        assert results.pop(key).body == path.read_text()
+    assert_parser_results(results, EXPECTED_OPEN_API_PATH / "openapi_parser_parse_alias")
 
 
 def test_openapi_parser_parse_modular(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with modular structure."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(Path(DATA_PATH / "modular.yaml"), data_model_field_type=DataModelFieldBase)
     modules = parser.parse()
-    main_modular_dir = EXPECTED_OPEN_API_PATH / "openapi_parser_parse_modular"
-
-    for paths, result in modules.items():
-        expected = main_modular_dir.joinpath(*paths).read_text()
-        assert result.body == expected
+    assert_parser_modules(modules, EXPECTED_OPEN_API_PATH / "openapi_parser_parse_modular")
 
 
 @pytest.mark.parametrize(
@@ -465,31 +498,34 @@ def test_openapi_parser_parse_modular(tmp_path: Path, monkeypatch: pytest.Monkey
     ],
 )
 def test_openapi_parser_parse_additional_properties(with_import: bool, format_: bool, base_class: str | None) -> None:
+    """Test parsing OpenAPI with additional properties."""
     parser = OpenAPIParser(
         Path(DATA_PATH / "additional_properties.yaml").read_text(encoding="utf-8"),
         base_class=base_class,
         data_model_field_type=DataModelFieldBase,
     )
 
-    assert (
-        parser.parse(with_import=with_import, format_=format_, settings_path=DATA_PATH.parent)
-        == get_expected_file(
+    assert_output(
+        parser.parse(with_import=with_import, format_=format_, settings_path=DATA_PATH.parent),
+        get_expected_file(
             "openapi_parser_parse_additional_properties",
             with_import,
             format_,
             base_class,
-        ).read_text()
+        ),
     )
 
 
 def test_openapi_parser_parse_array_enum(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with array enum types."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(source=Path(DATA_PATH / "array_enum.yaml"))
     expected_file = get_expected_file("openapi_parser_parse_array_enum", True, True)
-    assert parser.parse() == expected_file.read_text()
+    assert_output(parser.parse(), expected_file)
 
 
 def test_openapi_parser_parse_remote_ref(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with remote references."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
@@ -498,10 +534,18 @@ def test_openapi_parser_parse_remote_ref(tmp_path: Path, monkeypatch: pytest.Mon
     )
     expected_file = get_expected_file("openapi_parser_parse_remote_ref", True, True)
 
-    assert parser.parse() == expected_file.read_text()
+    assert_output(parser.parse(), expected_file)
+
+
+def test_openapi_parser_parse_required_null(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with required nullable fields."""
+    monkeypatch.chdir(tmp_path)
+    parser = OpenAPIParser(source=Path(DATA_PATH / "required_null.yaml"))
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_required_null" / "output.py")
 
 
 def test_openapi_model_resolver(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test OpenAPI model resolver functionality."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(source=(DATA_PATH / "api.yaml"))
     parser.parse()
@@ -594,15 +638,17 @@ def test_openapi_model_resolver(tmp_path: Path, monkeypatch: pytest.MonkeyPatch)
 
 
 def test_openapi_parser_parse_any(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI with any type schemas."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
         source=Path(DATA_PATH / "any.yaml"),
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_parse_any" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_parse_any" / "output.py")
 
 
 def test_openapi_parser_responses_without_content(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI responses without content."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
@@ -610,20 +656,18 @@ def test_openapi_parser_responses_without_content(tmp_path: Path, monkeypatch: p
         openapi_scopes=[OpenAPIScope.Paths],
         allow_responses_without_content=True,
     )
-    assert (
-        parser.parse()
-        == (EXPECTED_OPEN_API_PATH / "openapi_parser_responses_without_content" / "output.py").read_text()
-    )
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_responses_without_content" / "output.py")
 
 
 def test_openapi_parser_responses_with_tag(tmp_path: Path, monkeypatch: pytest.MonkeyPatch) -> None:
+    """Test parsing OpenAPI responses with tags."""
     monkeypatch.chdir(tmp_path)
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
         source=Path(DATA_PATH / "body_and_parameters.yaml"),
         openapi_scopes=[OpenAPIScope.Tags, OpenAPIScope.Schemas, OpenAPIScope.Paths],
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_responses_with_tag" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_responses_with_tag" / "output.py")
 
 
 @pytest.mark.skipif(
@@ -631,6 +675,7 @@ def test_openapi_parser_responses_with_tag(tmp_path: Path, monkeypatch: pytest.M
     reason="Installed black doesn't support the old style",
 )
 def test_openapi_parser_with_query_parameters() -> None:
+    """Test parsing OpenAPI with query parameters."""
     parser = OpenAPIParser(
         data_model_field_type=DataModelFieldBase,
         source=Path(DATA_PATH / "query_parameters.yaml"),
@@ -640,7 +685,42 @@ def test_openapi_parser_with_query_parameters() -> None:
             OpenAPIScope.Paths,
         ],
     )
-    assert parser.parse() == (EXPECTED_OPEN_API_PATH / "openapi_parser_with_query_parameters" / "output.py").read_text()
+    assert_output(parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_with_query_parameters" / "output.py")
+
+
+@pytest.mark.skipif(
+    black.__version__.split(".")[0] >= "24",
+    reason="Installed black doesn't support the old style",
+)
+def test_openapi_parser_with_include_path_parameters() -> None:
+    """Test parsing OpenAPI with included path parameters."""
+    parser = OpenAPIParser(
+        data_model_field_type=DataModelFieldBase,
+        source=Path(DATA_PATH / "query_parameters.yaml"),
+        openapi_scopes=[
+            OpenAPIScope.Parameters,
+            OpenAPIScope.Schemas,
+            OpenAPIScope.Paths,
+        ],
+        include_path_parameters=True,
+    )
+    assert_output(
+        parser.parse(), EXPECTED_OPEN_API_PATH / "openapi_parser_with_query_parameters" / "with_path_params.py"
+    )
+
+
+def test_parse_all_parameters_duplicate_names_exception() -> None:
+    """Test parsing parameters with duplicate names raises exception."""
+    parser = OpenAPIParser("", include_path_parameters=True)
+    parameters = [
+        ParameterObject.parse_obj({"name": "duplicate_param", "in": "path", "schema": {"type": "string"}}),
+        ParameterObject.parse_obj({"name": "duplicate_param", "in": "query", "schema": {"type": "integer"}}),
+    ]
+
+    with pytest.raises(Exception) as exc_info:  # noqa: PT011
+        parser.parse_all_parameters("TestModel", parameters, ["test", "path"])
+
+    assert "Parameter name 'duplicate_param' is used more than once." in str(exc_info.value)
 
 
 @pytest.mark.skipif(
@@ -648,6 +728,7 @@ def test_openapi_parser_with_query_parameters() -> None:
     reason="Require Pydantic version 2.0.0 or later ",
 )
 def test_openapi_parser_array_called_fields_with_one_of_items() -> None:
+    """Test parsing OpenAPI array fields with oneOf items."""
     parser = OpenAPIParser(
         data_model_field_type=DataModelField,
         source=Path(DATA_PATH / "array_called_fields_with_oneOf_items.yaml"),
@@ -658,11 +739,9 @@ def test_openapi_parser_array_called_fields_with_one_of_items() -> None:
         ],
         field_constraints=True,
     )
-    assert (
-        parser.parse()
-        == (
-            EXPECTED_OPEN_API_PATH / "openapi_parser_parse_array_called_fields_with_oneOf_items" / "output.py"
-        ).read_text()
+    assert_output(
+        parser.parse(),
+        EXPECTED_OPEN_API_PATH / "openapi_parser_parse_array_called_fields_with_oneOf_items" / "output.py",
     )
 
 
@@ -679,3 +758,158 @@ def test_no_additional_imports() -> None:
         source="",
     )
     assert len(new_parser.imports) == 0
+
+
+@pytest.mark.parametrize(
+    ("request_body_data", "expected_type_hints"),
+    [
+        pytest.param(
+            {"application/json": {"schema": {"type": "object", "properties": {"name": {"type": "string"}}}}},
+            {"application/json": "TestRequest"},
+            id="object_with_properties",
+        ),
+        pytest.param(
+            {
+                "application/json": {"schema": {"type": "object", "properties": {"name": {"type": "string"}}}},
+                "text/plain": {"schema": {"type": "string"}},
+            },
+            {"application/json": "TestRequest", "text/plain": "str"},
+            id="multiple_media_types",
+        ),
+        pytest.param(
+            {"application/json": {"schema": {"$ref": "#/components/schemas/RequestRef"}}},
+            {"application/json": "RequestRef"},
+            id="schema_reference",
+        ),
+        pytest.param(
+            {"application/json": {}},  # MediaObject with no schema
+            {},  # Should result in empty dict since no schema to process
+            id="missing_schema",
+        ),
+    ],
+)
+def test_parse_request_body_return(request_body_data: dict[str, Any], expected_type_hints: dict[str, str]) -> None:
+    """Test parsing request body returns correct type hints."""
+    parser = OpenAPIParser(
+        data_model_field_type=DataModelFieldBase,
+        source="",
+        use_standard_collections=True,
+    )
+    result = parser.parse_request_body(
+        "TestRequest",
+        RequestBodyObject(
+            content={
+                media_type: MediaObject.parse_obj(media_data) for media_type, media_data in request_body_data.items()
+            }
+        ),
+        ["test", "path"],
+    )
+
+    assert isinstance(result, dict)
+    assert len(result) == len(expected_type_hints)
+    for media_type, expected_hint in expected_type_hints.items():
+        assert media_type in result
+        assert result[media_type].type_hint == expected_hint
+
+
+@pytest.mark.parametrize(
+    ("parameters_data", "expected_type_hint"),
+    [
+        pytest.param([], None, id="no_parameters"),
+        pytest.param(
+            [{"name": "search", "in": "query", "required": False, "schema": {"type": "string"}}],
+            "TestParametersQuery",
+            id="with_query_parameters",
+        ),
+        pytest.param(
+            [{"name": "userId", "in": "path", "required": True, "schema": {"type": "string"}}],
+            None,
+            id="path_parameter_only",
+        ),
+    ],
+)
+def test_parse_all_parameters_return(parameters_data: list[dict[str, Any]], expected_type_hint: str | None) -> None:
+    """Test parsing parameters returns correct type hints."""
+    parser = OpenAPIParser(
+        data_model_field_type=DataModelFieldBase,
+        source="",
+        openapi_scopes=[OpenAPIScope.Parameters],
+    )
+    result = parser.parse_all_parameters(
+        "TestParametersQuery",
+        [ParameterObject.parse_obj(param_data) for param_data in parameters_data],
+        ["test", "path"],
+    )
+    if expected_type_hint is None:
+        assert result is None
+    else:
+        assert result is not None
+        assert result.type_hint == expected_type_hint
+
+
+@pytest.mark.parametrize(
+    ("responses_data", "expected_type_hints"),
+    [
+        pytest.param(
+            {
+                "200": {
+                    "description": "Success",
+                    "content": {"application/json": {"schema": {"type": "string"}}},
+                }
+            },
+            {"200": {"application/json": "str"}},
+            id="simple_response_with_schema",
+        ),
+        pytest.param(
+            {
+                "200": {
+                    "description": "Success",
+                    "content": {
+                        "application/json": {"schema": {"type": "object", "properties": {"name": {"type": "string"}}}},
+                        "text/plain": {"schema": {"type": "string"}},
+                    },
+                },
+                "400": {
+                    "description": "Bad Request",
+                    "content": {"text/plain": {"schema": {"type": "string"}}},
+                },
+            },
+            {"200": {"application/json": "TestResponse", "text/plain": "str"}, "400": {"text/plain": "str"}},
+            id="multiple_status_codes_and_content_types",
+        ),
+        pytest.param(
+            {
+                "200": {
+                    "description": "Success",
+                    "content": {"application/json": {}},  # Content but no schema
+                }
+            },
+            {},  # Should skip since no schema in content
+            id="response_with_no_schema",
+        ),
+    ],
+)
+def test_parse_responses_return(
+    responses_data: dict[str, dict[str, Any]],
+    expected_type_hints: dict[str, dict[str, str]],
+) -> None:
+    """Test parsing responses returns correct type hints."""
+    parser = OpenAPIParser(
+        data_model_field_type=DataModelFieldBase,
+        source="",
+    )
+
+    result = parser.parse_responses(
+        "TestResponse",
+        {status_code: ResponseObject.parse_obj(response_data) for status_code, response_data in responses_data.items()},
+        ["test", "path"],
+    )
+
+    assert isinstance(result, dict)
+    assert len(result) == len(expected_type_hints)
+    for status_code, expected_content_types in expected_type_hints.items():
+        assert status_code in result
+        assert len(result[status_code]) == len(expected_content_types)
+        for content_type, expected_type_hint in expected_content_types.items():
+            assert content_type in result[status_code]
+            assert result[status_code][content_type].type_hint == expected_type_hint
